@@ -34,12 +34,12 @@ static void    gossip_protocol_class_init       (GossipProtocolClass *klass);
 static void    gossip_protocol_init             (GossipProtocol      *protocol);
 
 enum {
-	LOGGED_IN,
-	LOGGED_OUT,
+	CONNECTING,
+	CONNECTED,
+	DISCONNECTING,
+	DISCONNECTED,
 	NEW_MESSAGE,
 	CONTACT_ADDED,
-	CONTACT_UPDATED,
-	CONTACT_PRESENCE_UPDATED,
 	CONTACT_REMOVED,
 	COMPOSING,
 
@@ -89,8 +89,8 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 	klass->register_account     = NULL;
 	klass->new_account          = NULL;
 
-	signals[LOGGED_IN] =
-		g_signal_new ("logged-in",
+	signals[CONNECTING] =
+		g_signal_new ("connecting",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
@@ -99,9 +99,28 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_ACCOUNT);
 
-	/* Maybe include a reason for disconnect? */
-	signals[LOGGED_OUT] =
-		g_signal_new ("logged-out",
+	signals[CONNECTED] =
+		g_signal_new ("connected",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, GOSSIP_TYPE_ACCOUNT);
+
+	signals[DISCONNECTING] =
+		g_signal_new ("disconnecting",
+			      G_TYPE_FROM_CLASS (klass),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      libgossip_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1, GOSSIP_TYPE_ACCOUNT);
+
+	signals[DISCONNECTED] =
+		g_signal_new ("disconnected",
 			      G_TYPE_FROM_CLASS (klass),
 			      G_SIGNAL_RUN_LAST,
 			      0,
@@ -130,26 +149,6 @@ gossip_protocol_class_init (GossipProtocolClass *klass)
 			      G_TYPE_NONE,
 			      1, GOSSIP_TYPE_CONTACT);
 
-	signals[CONTACT_UPDATED] =
-		g_signal_new ("contact-updated",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_CONTACT);
-
-	signals[CONTACT_PRESENCE_UPDATED] =
-		g_signal_new ("contact-presence-updated",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
-			      G_TYPE_NONE,
-			      1, GOSSIP_TYPE_CONTACT);
-
 	signals[CONTACT_REMOVED] =
 		g_signal_new ("contact-removed",
 			      G_TYPE_FROM_CLASS (klass),
@@ -653,11 +652,11 @@ gossip_protocol_get_version (GossipProtocol         *protocol,
 }
 
 void
-gossip_protocol_register_account (GossipProtocol          *protocol,
-				  GossipAccount           *account,
-				  GossipVCard             *vcard,
-				  GossipRegisterCallback   callback,
-				  gpointer                 user_data)
+gossip_protocol_register_account (GossipProtocol            *protocol,
+				  GossipAccount             *account,
+				  GossipVCard               *vcard,
+				  GossipResultErrorCallback  callback,
+				  gpointer                   user_data)
 {
 	GossipProtocolClass *klass;
 
@@ -684,6 +683,38 @@ gossip_protocol_register_cancel (GossipProtocol *protocol)
 	}
 }
 
+void
+gossip_protocol_change_password (GossipProtocol            *protocol,
+				 const gchar               *new_password,
+				 GossipResultErrorCallback  callback,
+				 gpointer                   user_data)
+{
+	GossipProtocolClass *klass;
+
+	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
+	g_return_if_fail (new_password != NULL);
+	g_return_if_fail (callback != NULL);
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->change_password) {
+		klass->change_password (protocol, new_password,
+					callback, user_data);
+	}
+}
+
+void
+gossip_protocol_change_password_cancel (GossipProtocol *protocol)
+{
+	GossipProtocolClass *klass;
+
+	g_return_if_fail (GOSSIP_IS_PROTOCOL (protocol));
+
+	klass = GOSSIP_PROTOCOL_GET_CLASS (protocol);
+	if (klass->change_password_cancel) {
+		klass->change_password_cancel (protocol);
+	}
+}
+
 GossipAccount *
 gossip_protocol_new_account (GossipProtocol *protocol)
 {
