@@ -20,21 +20,22 @@
 
 #include <config.h>
 
+#include "gossip-avatar.h"
 #include "gossip-vcard.h"
 
 #define GOSSIP_VCARD_GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_VCARD, GossipVCardPriv))
 
 typedef struct _GossipVCardPriv GossipVCardPriv;
 
-struct _GossipVCardPriv { 
-	gchar    *name;
-	gchar    *nickname;
-	gchar    *email;
-	gchar    *url;
-	gchar    *country;
-	gchar    *description;
-	gpointer *avatar;
-	guint     avatar_size;
+struct _GossipVCardPriv {
+	gchar        *name;
+	gchar        *nickname;
+	gchar        *birthday;
+	gchar        *email;
+	gchar        *url;
+	gchar        *country;
+	gchar        *description;
+	GossipAvatar *avatar;
 };
 
 static void vcard_finalize     (GObject      *object);
@@ -51,25 +52,22 @@ enum {
 	PROP_0,
 	PROP_NAME,
 	PROP_NICKNAME,
+	PROP_BIRTHDAY,
 	PROP_EMAIL,
 	PROP_URL,
 	PROP_COUNTRY,
 	PROP_DESCRIPTION,
 	PROP_AVATAR,
-	PROP_AVATAR_SIZE
 };
 
 G_DEFINE_TYPE (GossipVCard, gossip_vcard, G_TYPE_OBJECT);
 
-static gpointer parent_class = NULL;
-
 static void
 gossip_vcard_class_init (GossipVCardClass *class)
 {
 	GObjectClass *object_class;
 
 	object_class = G_OBJECT_CLASS (class);
-	parent_class = g_type_class_peek_parent (class);
 
 	object_class->finalize		= vcard_finalize;
 	object_class->get_property	= vcard_get_property;
@@ -89,6 +87,13 @@ gossip_vcard_class_init (GossipVCardClass *class)
 							      "The nickname field",
 							      NULL,
 							      G_PARAM_READWRITE));
+	g_object_class_install_property (object_class,
+					 PROP_BIRTHDAY,
+					 g_param_spec_string ("birthday",
+							      "Birthday field",
+							      "The birthday field",
+							      NULL,
+							      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class,
 					 PROP_EMAIL,
 					 g_param_spec_string ("email",
@@ -123,17 +128,7 @@ gossip_vcard_class_init (GossipVCardClass *class)
 							       "Avatar image",
 							       "The avatar image",
 							       G_PARAM_READWRITE));
-	
-	g_object_class_install_property (object_class,
-					 PROP_AVATAR_SIZE,
-					 g_param_spec_uint ("avatar_size",
-							    "Avatar size",
-							    "The size of the avatar image",
-							    0, 
-							    G_MAXUINT,
-							    0,
-							    G_PARAM_READWRITE));
-	
+
 	g_type_class_add_private (object_class, sizeof (GossipVCardPriv));
 }
 
@@ -146,30 +141,34 @@ gossip_vcard_init (GossipVCard *vcard)
 
 	priv->name        = NULL;
 	priv->nickname    = NULL;
+	priv->birthday    = NULL;
 	priv->email       = NULL;
 	priv->url         = NULL;
-        priv->country     = NULL;
+	priv->country     = NULL;
 	priv->description = NULL;
 	priv->avatar      = NULL;
-	priv->avatar_size = 0;
 }
 
-static void                
+static void
 vcard_finalize (GObject *object)
 {
 	GossipVCardPriv *priv;
 
 	priv = GOSSIP_VCARD_GET_PRIV (object);
-	
+
 	g_free (priv->name);
 	g_free (priv->nickname);
+	g_free (priv->birthday);
 	g_free (priv->email);
 	g_free (priv->url);
 	g_free (priv->country);
 	g_free (priv->description);
-	g_free (priv->avatar);
-	
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+
+	if (priv->avatar) {
+		gossip_avatar_unref (priv->avatar);
+	}
+
+	(G_OBJECT_CLASS (gossip_vcard_parent_class)->finalize) (object);
 }
 
 static void
@@ -189,6 +188,9 @@ vcard_get_property (GObject    *object,
 	case PROP_NICKNAME:
 		g_value_set_string (value, priv->nickname);
 		break;
+	case PROP_BIRTHDAY:
+		g_value_set_string (value, priv->birthday);
+		break;
 	case PROP_EMAIL:
 		g_value_set_string (value, priv->email);
 		break;
@@ -204,13 +206,10 @@ vcard_get_property (GObject    *object,
 	case PROP_AVATAR:
 		g_value_set_pointer (value, priv->avatar);
 		break;
-	case PROP_AVATAR_SIZE:
-		g_value_set_uint (value, priv->avatar_size);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
-	};		
+	};
 }
 
 static void
@@ -225,19 +224,23 @@ vcard_set_property (GObject      *object,
 
 	switch (param_id) {
 	case PROP_NAME:
-		gossip_vcard_set_name (GOSSIP_VCARD (object), 
+		gossip_vcard_set_name (GOSSIP_VCARD (object),
 				       g_value_get_string (value));
 		break;
 	case PROP_NICKNAME:
-		gossip_vcard_set_nickname (GOSSIP_VCARD (object), 
+		gossip_vcard_set_nickname (GOSSIP_VCARD (object),
+					   g_value_get_string (value));
+		break;
+	case PROP_BIRTHDAY:
+		gossip_vcard_set_birthday (GOSSIP_VCARD (object),
 					   g_value_get_string (value));
 		break;
 	case PROP_EMAIL:
-		gossip_vcard_set_email (GOSSIP_VCARD (object), 
+		gossip_vcard_set_email (GOSSIP_VCARD (object),
 					g_value_get_string (value));
 		break;
 	case PROP_URL:
-		gossip_vcard_set_url (GOSSIP_VCARD (object), 
+		gossip_vcard_set_url (GOSSIP_VCARD (object),
 				      g_value_get_string (value));
 		break;
 	case PROP_COUNTRY:
@@ -245,19 +248,17 @@ vcard_set_property (GObject      *object,
 					  g_value_get_string (value));
 		break;
 	case PROP_DESCRIPTION:
-		gossip_vcard_set_description (GOSSIP_VCARD (object), 
+		gossip_vcard_set_description (GOSSIP_VCARD (object),
 					      g_value_get_string (value));
 		break;
 	case PROP_AVATAR:
-		priv->avatar = g_value_get_pointer (value);
-		break;
-	case PROP_AVATAR_SIZE:
-		priv->avatar_size = g_value_get_uint (value);
+		gossip_vcard_set_avatar (GOSSIP_VCARD (object),
+					 g_value_get_pointer (value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
-	};		
+	};
 }
 
 GossipVCard *
@@ -272,24 +273,36 @@ gossip_vcard_get_name (GossipVCard *vcard)
 	GossipVCardPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	return priv->name;
 }
 
 const gchar *
-gossip_vcard_get_nickname (GossipVCard *vcard) 
+gossip_vcard_get_nickname (GossipVCard *vcard)
 {
 	GossipVCardPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	return priv->nickname;
 }
 
+const gchar *
+gossip_vcard_get_birthday (GossipVCard *vcard)
+{
+	GossipVCardPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
+
+	priv = GOSSIP_VCARD_GET_PRIV (vcard);
+
+	return priv->birthday;
+}
+
 const gchar *
 gossip_vcard_get_email (GossipVCard *vcard)
 {
@@ -308,7 +321,7 @@ gossip_vcard_get_url (GossipVCard *vcard)
 	GossipVCardPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	return priv->url;
@@ -320,7 +333,7 @@ gossip_vcard_get_country (GossipVCard *vcard)
 	GossipVCardPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	return priv->country;
@@ -332,40 +345,34 @@ gossip_vcard_get_description (GossipVCard *vcard)
 	GossipVCardPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	return priv->description;
 }
 
-const guchar *
-gossip_vcard_get_avatar (GossipVCard *vcard, 
-			 gsize       *avatar_size)
+GossipAvatar *
+gossip_vcard_get_avatar (GossipVCard *vcard)
 {
 	GossipVCardPriv *priv;
-	guchar          *avatar;
 
 	g_return_val_if_fail (GOSSIP_IS_VCARD (vcard), NULL);
-	g_return_val_if_fail (avatar_size != NULL, NULL);
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
-	avatar = (guchar*) priv->avatar;
-	*avatar_size = priv->avatar_size;
-
-	return avatar;
+	return priv->avatar;
 }
 
 
 
 void
-gossip_vcard_set_name (GossipVCard *vcard, 
+gossip_vcard_set_name (GossipVCard *vcard,
 		       const gchar *name)
 {
 	GossipVCardPriv *priv;
 
 	g_return_if_fail (GOSSIP_IS_VCARD (vcard));
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	g_free (priv->name);
@@ -378,7 +385,7 @@ gossip_vcard_set_name (GossipVCard *vcard,
 }
 
 void
-gossip_vcard_set_nickname (GossipVCard *vcard, 
+gossip_vcard_set_nickname (GossipVCard *vcard,
 			   const gchar *nickname)
 {
 	GossipVCardPriv *priv;
@@ -395,6 +402,24 @@ gossip_vcard_set_nickname (GossipVCard *vcard,
 	}
 }
 
+void
+gossip_vcard_set_birthday (GossipVCard *vcard,
+			   const gchar *birthday)
+{
+	GossipVCardPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_VCARD (vcard));
+
+	priv = GOSSIP_VCARD_GET_PRIV (vcard);
+
+	g_free (priv->birthday);
+	if (birthday) {
+		priv->birthday = g_strdup (birthday);
+	} else {
+		priv->birthday = NULL;
+	}
+}
+
 void
 gossip_vcard_set_email (GossipVCard *vcard,
 			const gchar *email)
@@ -414,14 +439,14 @@ gossip_vcard_set_email (GossipVCard *vcard,
 	}
 }
 
-void   
+void
 gossip_vcard_set_url (GossipVCard *vcard,
 		      const gchar *url)
 {
 	GossipVCardPriv *priv;
 
 	g_return_if_fail (GOSSIP_IS_VCARD (vcard));
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	g_free (priv->url);
@@ -433,14 +458,14 @@ gossip_vcard_set_url (GossipVCard *vcard,
 	}
 }
 
-void   
-gossip_vcard_set_country (GossipVCard *vcard, 
+void
+gossip_vcard_set_country (GossipVCard *vcard,
 			  const gchar *country)
 {
 	GossipVCardPriv *priv;
 
 	g_return_if_fail (GOSSIP_IS_VCARD (vcard));
-	
+
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
 
 	g_free (priv->country);
@@ -471,24 +496,23 @@ gossip_vcard_set_description (GossipVCard *vcard,
 	}
 }
 
-void 
-gossip_vcard_set_avatar (GossipVCard  *vcard, 
-			 const guchar *avatar, 
-			 gsize         avatar_size)
+void
+gossip_vcard_set_avatar (GossipVCard  *vcard,
+			 GossipAvatar *avatar)
 {
 	GossipVCardPriv *priv;
 
 	g_return_if_fail (GOSSIP_IS_VCARD (vcard));
 
 	priv = GOSSIP_VCARD_GET_PRIV (vcard);
-	
-	g_free (priv->avatar);
+
+	if (priv->avatar) {
+		gossip_avatar_unref (priv->avatar);
+		priv->avatar = NULL;
+	}
 
 	if (avatar) {
-		priv->avatar = g_memdup (avatar, avatar_size);
-		priv->avatar_size = avatar_size;
-	} else {
-		priv->avatar = NULL;
-		priv->avatar_size = 0;
+		priv->avatar = gossip_avatar_ref (avatar);
 	}
 }
+
