@@ -25,6 +25,7 @@
 
 #include <loudmouth/loudmouth.h>
 
+#include <libgossip/gossip-avatar.h>
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-utils.h>
 
@@ -115,6 +116,11 @@ jabber_vcard_get_cb (LmMessageHandler   *handler,
 		gossip_vcard_set_nickname (vcard, node->value);
 	}
 
+	node = lm_message_node_get_child (vcard_node, "BDAY");
+	if (node) {
+		gossip_vcard_set_birthday (vcard, node->value);
+	}
+
 	node = lm_message_node_get_child (vcard_node, "EMAIL");
 	if (node) {
 		const gchar *email = NULL;
@@ -153,12 +159,16 @@ jabber_vcard_get_cb (LmMessageHandler   *handler,
 	if (photo_node) {
 		node = lm_message_node_get_child (photo_node, "BINVAL");
 		if (node && node->value) {
-			guchar *decoded_avatar;
-			gsize   len;
+			guchar       *decoded_avatar;
+			GossipAvatar *avatar;
+			gsize         len;
 
 			decoded_avatar = g_base64_decode (node->value, &len);
-			gossip_vcard_set_avatar (vcard, decoded_avatar, len);
+			avatar = gossip_avatar_new (decoded_avatar,
+						    len, "image/png");
+			gossip_vcard_set_avatar (vcard, avatar);
 			g_free (decoded_avatar);
+			gossip_avatar_unref (avatar);
 		}
 	}
 
@@ -169,6 +179,12 @@ jabber_vcard_get_cb (LmMessageHandler   *handler,
 	return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 }
 
+static void
+jabber_vcard_free (GossipCallbackData *data)
+{
+	g_slice_free (GossipCallbackData, data);
+}
+
 gboolean
 gossip_jabber_vcard_get (GossipJabber         *jabber,
 			 const gchar          *jid_str,
@@ -199,13 +215,13 @@ gossip_jabber_vcard_get (GossipJabber         *jabber,
 	node = lm_message_node_add_child (m->node, "vCard", NULL);
 	lm_message_node_set_attribute (node, "xmlns", "vcard-temp");
 
-	data = g_new0 (GossipCallbackData, 1);
+	data = g_slice_new0 (GossipCallbackData);
 	data->callback = callback;
 	data->user_data = user_data;
 
 	handler = lm_message_handler_new ((LmHandleMessageFunction) jabber_vcard_get_cb,
 					  data,
-					  g_free);
+					  (GDestroyNotify) jabber_vcard_free);
 
 	if (!lm_connection_send_with_reply (connection, m, handler, error)) {
 		lm_message_unref (m);
@@ -257,9 +273,9 @@ gossip_jabber_vcard_set (GossipJabber          *jabber,
 	LmMessageNode      *child;
 	LmMessageHandler   *handler;
 	GossipCallbackData *data;
+	GossipAvatar       *avatar;
+	const gchar        *str;
 	gboolean            result;
-	const guchar       *avatar;
-	gsize		    avatar_size;
 
 	connection = gossip_jabber_get_connection (jabber);
 
@@ -270,25 +286,45 @@ gossip_jabber_vcard_set (GossipJabber          *jabber,
 	node = lm_message_node_add_child (m->node, "vCard", NULL);
 	lm_message_node_set_attribute (node, "xmlns", "vcard-temp");
 
-	lm_message_node_add_child (node, "FN", gossip_vcard_get_name (vcard));
-	lm_message_node_add_child (node, "NICKNAME",
-				   gossip_vcard_get_nickname (vcard));
-	lm_message_node_add_child (node, "URL", gossip_vcard_get_url (vcard));
+	str = gossip_vcard_get_name (vcard);
+	if (!G_STR_EMPTY (str)) {
+		lm_message_node_add_child (node, "FN", str);
+	}
 
-	child = lm_message_node_add_child (node, "EMAIL", NULL);
-	lm_message_node_add_child (child, "USERID", gossip_vcard_get_email (vcard));
+	str = gossip_vcard_get_nickname (vcard);
+	if (!G_STR_EMPTY (str)) {
+		lm_message_node_add_child (node, "NICKNAME", str);
+	}
 
-	lm_message_node_add_child (node, "DESC",
-				   gossip_vcard_get_description (vcard));
+	str = gossip_vcard_get_birthday (vcard);
+	if (!G_STR_EMPTY (str)) {
+		lm_message_node_add_child (node, "BDAY", str);
+	}
 
-	avatar = gossip_vcard_get_avatar (vcard, &avatar_size);
+	str = gossip_vcard_get_url (vcard);
+	if (!G_STR_EMPTY (str)) {
+		lm_message_node_add_child (node, "URL", str);
+	}
+	
+	str = gossip_vcard_get_email (vcard);
+	if (!G_STR_EMPTY (str)) {
+		child = lm_message_node_add_child (node, "EMAIL", NULL);
+		lm_message_node_add_child (child, "USERID", str);
+	}
+
+	str = gossip_vcard_get_description (vcard);
+	if (!G_STR_EMPTY (str)) {
+		lm_message_node_add_child (node, "DESC", str);
+	}
+
+	avatar = gossip_vcard_get_avatar (vcard);
 	if (avatar != NULL) {
 		gchar *avatar_encoded;
 
 		node = lm_message_node_add_child (node, "PHOTO", NULL);
-		lm_message_node_add_child (node, "TYPE", "image/png");
+		lm_message_node_add_child (node, "TYPE", avatar->format);
 
-		avatar_encoded = g_base64_encode (avatar, avatar_size);
+		avatar_encoded = g_base64_encode (avatar->data, avatar->len);
 		lm_message_node_add_child (node, "BINVAL", avatar_encoded);
 		g_free (avatar_encoded);
 	}
