@@ -28,6 +28,7 @@
 
 #include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-conf.h>
+#include <libgossip/gossip-log.h>
 #include <libgossip/gossip-protocol.h>
 
 #include "gossip-add-contact-dialog.h"
@@ -35,8 +36,8 @@
 #include "gossip-chat-invite.h"
 #include "gossip-chat-window.h"
 #include "gossip-contact-info-dialog.h"
-#include "gossip-log.h"
 #include "gossip-log-window.h"
+#include "gossip-new-chatroom-dialog.h"
 #include "gossip-preferences.h"
 #include "gossip-private-chat.h"
 #include "gossip-sound.h"
@@ -77,6 +78,8 @@ struct _GossipChatWindowPriv {
 	GtkWidget   *menu_conv_close;
 
 	GtkWidget   *menu_room;
+	GtkWidget   *menu_room_set_topic;
+	GtkWidget   *menu_room_join_new;
 	GtkWidget   *menu_room_invite;
 	GtkWidget   *menu_room_add;
 	GtkWidget   *menu_room_show_contacts;
@@ -138,6 +141,10 @@ static void       chat_window_insert_smiley_activate_cb (GtkWidget             *
 							 GossipChatWindow      *window);
 static void       chat_window_close_activate_cb         (GtkWidget             *menuitem,
 							 GossipChatWindow      *window);
+static void       chat_window_room_set_topic_activate_cb(GtkWidget             *menuitem,
+							 GossipChatWindow      *window);
+static void       chat_window_room_join_new_activate_cb (GtkWidget             *menuitem,
+							 GossipChatWindow      *window);
 static void       chat_window_room_invite_activate_cb   (GtkWidget             *menuitem,
 							 GossipChatWindow      *window);
 static void       chat_window_room_add_activate_cb      (GtkWidget             *menuitem,
@@ -172,27 +179,27 @@ static void       chat_window_new_message_cb            (GossipChat            *
 							 GossipChatWindow      *window);
 static void       chat_window_disconnected_cb           (GossipApp             *app,
 							 GossipChatWindow      *window);
-static void       chat_window_switch_page_cb            (GtkNotebook           *notebook,
-							 GtkNotebookPage       *page,
-							 gint                   page_num,
-							 GossipChatWindow      *window);
 static GtkNotebook* chat_window_detach_hook             (GtkNotebook           *source,
 							 GtkWidget             *page,
 							 gint                   x,
 							 gint                   y,
 							 gpointer               user_data);
+static void       chat_window_page_switched_cb          (GtkNotebook           *notebook,
+							 GtkNotebookPage       *page,
+							 gint                   page_num,
+							 GossipChatWindow      *window);
+static void       chat_window_page_reordered_cb         (GtkNotebook           *notebook,
+							 GtkWidget             *widget,
+							 guint                  page_num,
+							 GossipChatWindow      *window);
 static void       chat_window_page_added_cb             (GtkNotebook           *notebook,
 							 GtkWidget             *child,
 							 guint                  page_num,
 							 GossipChatWindow      *window);
-static void       chat_window_page_removed_cb            (GtkNotebook           *notebook,
+static void       chat_window_page_removed_cb           (GtkNotebook           *notebook,
 							 GtkWidget             *child,
 							 guint                  page_num,
 							 GossipChatWindow      *window);
-static void       chat_window_page_reordered_cb         (GtkNotebook           *notebook,
-							 GtkWidget             *widget,
-							 guint                  page_num,
-							 GossipChatWindow      *window);
 static gboolean   chat_window_focus_in_event_cb         (GtkWidget             *widget,
 							 GdkEvent              *event,
 							 GossipChatWindow      *window);
@@ -281,6 +288,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 				       "menu_conv_info", &priv->menu_conv_info,
 				       "menu_conv_close", &priv->menu_conv_close,
 				       "menu_room", &priv->menu_room,
+				       "menu_room_set_topic", &priv->menu_room_set_topic,
+				       "menu_room_join_new", &priv->menu_room_join_new,
 				       "menu_room_invite", &priv->menu_room_invite,
 				       "menu_room_add", &priv->menu_room_add,
 				       "menu_room_show_contacts", &priv->menu_room_show_contacts,
@@ -303,6 +312,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 			      "menu_conv_add_contact", "activate", chat_window_add_contact_activate_cb,
 			      "menu_conv_info", "activate", chat_window_info_activate_cb,
 			      "menu_conv_close", "activate", chat_window_close_activate_cb,
+			      "menu_room_set_topic", "activate", chat_window_room_set_topic_activate_cb,
+			      "menu_room_join_new", "activate", chat_window_room_join_new_activate_cb,
 			      "menu_room_invite", "activate", chat_window_room_invite_activate_cb,
 			      "menu_room_add", "activate", chat_window_room_add_activate_cb,
 			      "menu_edit", "activate", chat_window_edit_activate_cb,
@@ -386,7 +397,7 @@ gossip_chat_window_init (GossipChatWindow *window)
 			  window);
 	g_signal_connect_after (priv->notebook,
 				"switch_page",
-				G_CALLBACK (chat_window_switch_page_cb),
+				G_CALLBACK (chat_window_page_switched_cb),
 				window);
 	g_signal_connect (priv->notebook,
 			  "page_reordered",
@@ -600,8 +611,8 @@ chat_window_create_label (GossipChatWindow *window,
 
 	name = gossip_chat_get_name (chat);
 	name_label = gtk_label_new (name);
+ 	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END);
 
-/* 	gtk_label_set_ellipsize (GTK_LABEL (name_label), PANGO_ELLIPSIZE_END); */
 
 	attr_list = pango_attr_list_new ();
 	attr = pango_attr_scale_new (1/1.2);
@@ -786,6 +797,7 @@ chat_window_update_menu (GossipChatWindow *window)
 
 		gtk_widget_set_sensitive (priv->menu_room_add, !saved);
 		gtk_widget_set_sensitive (priv->menu_conv_insert_smiley, is_connected);
+		gtk_widget_set_sensitive (priv->menu_room_join_new, is_connected);
 		gtk_widget_set_sensitive (priv->menu_room_invite, is_connected);
 
 		/* We need to block the signal here because all we are
@@ -1018,6 +1030,33 @@ chat_window_close_activate_cb (GtkWidget        *menuitem,
 	gossip_chat_window_remove_chat (window, priv->current_chat);
 }
 
+static void
+chat_window_room_set_topic_activate_cb (GtkWidget        *menuitem,
+					GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	
+	priv = GET_PRIV (window);
+
+	if (gossip_chat_is_group_chat (priv->current_chat)) {
+		GossipGroupChat *group_chat;
+
+		group_chat = GOSSIP_GROUP_CHAT (priv->current_chat);
+		gossip_group_chat_set_topic (group_chat);
+	}
+}
+
+static void
+chat_window_room_join_new_activate_cb (GtkWidget        *menuitem,
+				       GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+
+	priv = GET_PRIV (window);
+
+	gossip_new_chatroom_dialog_show (GTK_WINDOW (priv->dialog));
+}
+
 static void
 chat_window_room_invite_activate_cb (GtkWidget        *menuitem,
 				     GossipChatWindow *window)
@@ -1369,38 +1408,6 @@ chat_window_disconnected_cb (GossipApp        *app,
 	chat_window_update_menu (window);
 }
 
-static void
-chat_window_switch_page_cb (GtkNotebook	     *notebook,
-			    GtkNotebookPage  *page,
-			    gint	      page_num,
-			    GossipChatWindow *window)
-{
-	GossipChatWindowPriv *priv;
-	GossipChat           *chat;
-	GtkWidget            *child;
-
-	gossip_debug (DEBUG_DOMAIN, "Page switched");
-
-	priv = GET_PRIV (window);
-
-	child = gtk_notebook_get_nth_page (notebook, page_num);
-	chat = g_object_get_data (G_OBJECT (child), "chat");
-
-	if (priv->page_added) {
-		priv->page_added = FALSE;
-		gossip_chat_scroll_down (chat);
-	} else if (priv->current_chat == chat) {
-		return;
-	}
-	
-	priv->current_chat = chat;
-	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
-
-	chat_window_update_title (window, NULL);
-	chat_window_update_menu (window);
-	chat_window_update_status (window, chat);
-}
-
 static GtkNotebook *
 chat_window_detach_hook (GtkNotebook *source,
 			 GtkWidget   *page,
@@ -1428,6 +1435,49 @@ chat_window_detach_hook (GtkNotebook *source,
 	return NULL;
 }
 
+static void
+chat_window_page_switched_cb (GtkNotebook	     *notebook,
+			      GtkNotebookPage  *page,
+			      gint	      page_num,
+			      GossipChatWindow *window)
+{
+	GossipChatWindowPriv *priv;
+	GossipChat           *chat;
+	GtkWidget            *child;
+
+	gossip_debug (DEBUG_DOMAIN, "Page switched");
+
+	priv = GET_PRIV (window);
+
+	child = gtk_notebook_get_nth_page (notebook, page_num);
+	chat = g_object_get_data (G_OBJECT (child), "chat");
+
+	if (priv->page_added) {
+		priv->page_added = FALSE;
+		gossip_chat_scroll_down (chat);
+	} else if (priv->current_chat == chat) {
+		return;
+	}
+
+	priv->current_chat = chat;
+	priv->chats_new_msg = g_list_remove (priv->chats_new_msg, chat);
+
+	chat_window_update_title (window, NULL);
+	chat_window_update_menu (window);
+	chat_window_update_status (window, chat);
+}
+
+static void
+chat_window_page_reordered_cb (GtkNotebook      *notebook,
+			       GtkWidget        *widget,
+			       guint             page_num,
+			       GossipChatWindow *window)
+{
+	gossip_debug (DEBUG_DOMAIN, "Page reordered");
+	
+	chat_window_update_menu (window);
+}
+
 static void
 chat_window_page_added_cb (GtkNotebook      *notebook,
 			   GtkWidget	    *child,
@@ -1554,17 +1604,6 @@ chat_window_page_removed_cb (GtkNotebook      *notebook,
 	}
 }
 
-static void
-chat_window_page_reordered_cb (GtkNotebook      *notebook,
-			       GtkWidget        *widget,
-			       guint             page_num,
-			       GossipChatWindow *window)
-{
-	gossip_debug (DEBUG_DOMAIN, "Page reordered");
-	
-	chat_window_update_menu (window);
-}
-
 static gboolean
 chat_window_focus_in_event_cb (GtkWidget        *widget,
 			       GdkEvent         *event,
@@ -1572,10 +1611,14 @@ chat_window_focus_in_event_cb (GtkWidget        *widget,
 {
 	GossipChatWindowPriv *priv;
 
-	priv = GET_PRIV (window);
+	return FALSE;
 
+	gossip_debug (DEBUG_DOMAIN, "Focus in event, updating title");
+
+	priv = GET_PRIV (window);
 	priv->new_msg = FALSE;
 
+	/* Set the title, since we now mark all unread messages as read */
 	chat_window_update_title (window, NULL);
 
 	return FALSE;
@@ -1764,14 +1807,14 @@ gossip_chat_window_add_chat (GossipChatWindow *window,
 
 		gossip_chat_load_geometry (chat, &x, &y, &w, &h);
 
-		if (x >= 1 && y >= 1) {
+		if (x >= 0 && y >= 0) {
 			/* Let the window manager position it if we
 			 * don't have good x, y coordinates.
 			 */
 			gtk_window_move (GTK_WINDOW (priv->dialog), x, y);
 		}
 
-		if (w >= 1 && h >= 1) {
+		if (w >= 0 && h >= 0) {
 			/* Use the defaults from the glade file if we
 			 * don't have good w, h geometry.
 			 */
@@ -1785,7 +1828,9 @@ gossip_chat_window_add_chat (GossipChatWindow *window,
 	gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), child, label);
 	gtk_notebook_set_tab_reorderable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
 	gtk_notebook_set_tab_detachable (GTK_NOTEBOOK (priv->notebook), child, TRUE);
-
+	gtk_notebook_set_tab_label_packing (GTK_NOTEBOOK (priv->notebook), child,
+					    TRUE, TRUE, GTK_PACK_START); 
+	
 	gossip_debug (DEBUG_DOMAIN, 
 		      "Chat added (%d references)",
 		      G_OBJECT (chat)->ref_count);
