@@ -27,6 +27,7 @@
 #include <libgossip/gossip-contact.h>
 #include <libgossip/gossip-session.h>
 #include <libgossip/gossip-paths.h>
+#include <libgossip/gossip-utils.h>
 
 #include "gossip-app.h"
 #include "gossip-avatar-image.h"
@@ -42,6 +43,7 @@ typedef struct {
 	GtkWidget     *name_label;
 	GtkWidget     *stub_email_label;
 	GtkWidget     *stub_web_label;
+	GtkWidget     *stub_birthday_label;
 	GtkWidget     *personal_status_label;
 	GtkWidget     *personal_status_hbox;
 	GtkWidget     *personal_vbox;
@@ -60,8 +62,6 @@ typedef struct {
 	GtkWidget     *client_not_avail_label;
 	GtkWidget     *client_table;
 
-	GtkWidget     *subscription_hbox;
-	GtkWidget     *subscription_label;
 	GtkWidget     *presence_list_vbox;
 	GtkWidget     *presence_table;
 
@@ -69,14 +69,12 @@ typedef struct {
 	GtkWidget     *description_label;
 	GtkWidget     *avatar_image;
 
-	gulong         contact_signal_handler;
 	gulong         presence_signal_handler;
 
 	guint          show_timeout_id;
 } GossipContactInfoDialog;
 
 static void contact_info_dialog_init                 (void);
-static void contact_info_dialog_update_subscription  (GossipContactInfoDialog *dialog);
 static void contact_info_dialog_update_presences     (GossipContactInfoDialog *dialog);
 static void contact_info_dialog_get_vcard_cb         (GossipResult             result,
 						      GossipVCard             *vcard,
@@ -84,55 +82,38 @@ static void contact_info_dialog_get_vcard_cb         (GossipResult             r
 static void contact_info_dialog_get_version_cb       (GossipResult             result,
 						      GossipVersionInfo       *version_info,
 						      GossipContact           *contact);
-static void contact_info_dialog_contact_updated_cb   (GossipSession           *session,
-						      GossipContact           *contact,
+static void contact_info_dialog_presence_updated_cb  (GossipContact           *contact,
+						      GParamSpec              *param,
 						      gpointer                 user_data);
-static void contact_info_dialog_presence_updated_cb  (GossipSession           *session,
-						      GossipContact           *contact,
-						      gpointer                 user_data);
-static void contact_info_dialog_subscribe_clicked_cb (GtkWidget               *widget,
-						      GossipContactInfoDialog *dialog);
 static void contact_info_dialog_destroy_cb           (GtkWidget               *widget,
 						      GossipContactInfoDialog *dialog);
 static void contact_info_dialog_response_cb          (GtkWidget               *widget,
 						      gint                     response,
 						      GossipContactInfoDialog *dialog);
 
-static GHashTable *contact_info_dialogs = NULL;
+static GHashTable *dialogs = NULL;
 
 static void
 contact_info_dialog_init (void)
 {
-	static gboolean inited = FALSE;
-
-	if (inited) {
+	if (dialogs) {
 		return;
 	}
-	
-	inited = TRUE;
-	
-	contact_info_dialogs = g_hash_table_new_full (gossip_contact_hash,
-						      gossip_contact_equal,
-						      g_object_unref,
-						      NULL);
-}
 
-static void
-contact_info_dialog_update_subscription (GossipContactInfoDialog *dialog)
-{
-	GossipSubscription subscription;
+	dialogs = g_hash_table_new_full (gossip_contact_hash,
+					 gossip_contact_equal,
+					 g_object_unref,
+					 NULL);
 
-	g_return_if_fail (dialog != NULL);
-	g_return_if_fail (GOSSIP_IS_CONTACT (dialog->contact));
-
-	subscription = gossip_contact_get_subscription (dialog->contact);
-
-	if (subscription == GOSSIP_SUBSCRIPTION_NONE ||
-	    subscription == GOSSIP_SUBSCRIPTION_FROM) {
-		gtk_widget_show_all (dialog->subscription_hbox);
-	} else {
-		gtk_widget_hide (dialog->subscription_hbox);
-	}
+	/* Set up a style for the close button with no focus padding. */
+	gtk_rc_parse_string (
+		"style \"gossip-contact-info-link-button-style\"\n"
+		"{\n"
+		"  GtkWidget::focus-padding = 0\n"
+		"  xthickness = 0\n"
+		"  ythickness = 0\n"
+		"}\n"
+		"widget \"*.gossip-contact-info-link-button\" style \"gossip-contact-info-link-button-style\"");
 }
 
 static void
@@ -155,7 +136,7 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 	   each resource if in the case of Jabber). */
 	dialog->presence_table = gtk_table_new (1, cols, FALSE);
 	gtk_widget_show (dialog->presence_table);
-	gtk_box_pack_start (GTK_BOX (dialog->presence_list_vbox), 
+	gtk_box_pack_start (GTK_BOX (dialog->presence_list_vbox),
 			    dialog->presence_table, FALSE, FALSE, 0);
 	gtk_table_set_row_spacings (GTK_TABLE (dialog->presence_table), 6);
 	gtk_table_set_col_spacings (GTK_TABLE (dialog->presence_table), 6);
@@ -171,18 +152,18 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 		widget = gtk_image_new_from_pixbuf (pixbuf);
 		g_object_unref (pixbuf);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
-				  widget, 
+				  widget,
 				  0, 1,
-				  i, i + 1, 
-				  0, 0, 
+				  i, i + 1,
+				  0, 0,
 				  0, 0);
 
 		widget = gtk_label_new (status);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
-				  widget, 
+				  widget,
 				  1, 2,
-				  i, i + 1, 
+				  i, i + 1,
 				  0, 0,
 				  0, 0);
 
@@ -193,23 +174,23 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 	for (l = presences, i = 0; l; l = l->next, i++) {
 		GossipPresence *presence;
 		GtkWidget      *widget;
-		
+
 		presence = l->data;
 
 		if (presence) {
 			pixbuf = gossip_pixbuf_for_presence (presence);
-			
+
 			status = gossip_presence_get_status (presence);
 			if (!status) {
 				GossipPresenceState state;
-				
+
 				state = gossip_presence_get_state (presence);
 				status = gossip_presence_state_get_default_status (state);
 			}
-			
+
 			resource = gossip_presence_get_resource (presence);
 		}
-		
+
 		if (i > 1) {
 			gtk_table_resize (GTK_TABLE (dialog->presence_table), i, cols);
 		}
@@ -217,20 +198,20 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 		widget = gtk_image_new_from_pixbuf (pixbuf);
 		g_object_unref (pixbuf);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
-				  widget, 
+				  widget,
 				  0, 1,
 				  i, i + 1,
-				  0, 0, 
+				  0, 0,
 				  0, 0);
-		
+
 		widget = gtk_label_new (status);
 		gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
-				  widget, 
+				  widget,
 				  1, 2,
-				  i, i + 1, 
-				  GTK_FILL, 
+				  i, i + 1,
+				  GTK_FILL,
 				  GTK_FILL,
 				  0, 0);
 
@@ -240,8 +221,8 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 
 		widget = gtk_label_new ("-");
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
-		gtk_table_attach (GTK_TABLE (dialog->presence_table), widget, 
-				  2, 3, 
+		gtk_table_attach (GTK_TABLE (dialog->presence_table), widget,
+				  2, 3,
 				  i, i + 1,
 				  0, 0,
 				  0, 0.5);
@@ -250,10 +231,10 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 		gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 		gtk_misc_set_alignment (GTK_MISC (widget), 0, 0.5);
 		gtk_table_attach (GTK_TABLE (dialog->presence_table),
-				  widget, 
+				  widget,
 				  3, 4,
 				  i, i + 1,
-				  GTK_EXPAND | GTK_FILL, 
+				  GTK_EXPAND | GTK_FILL,
 				  GTK_EXPAND | GTK_FILL,
 				  0, 0);
 	}
@@ -270,10 +251,10 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	gboolean                 show_personal = FALSE;
 	const gchar             *str;
 	GdkPixbuf               *pixbuf;
-	
-	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
+
+	dialog = g_hash_table_lookup (dialogs, contact);
 	g_object_unref (contact);
-	
+
 	if (!dialog) {
 		return;
 	}
@@ -286,28 +267,28 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 
 		return;
 	}
-	
+
 	pixbuf = gossip_pixbuf_avatar_from_vcard (vcard);
 	if (pixbuf != NULL) {
 		gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (dialog->avatar_image), pixbuf);
 		g_object_unref (pixbuf);
 	}
-	
+
 	str = gossip_vcard_get_description (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		gtk_label_set_text (GTK_LABEL (dialog->description_label), str);
 		gtk_widget_show (dialog->description_vbox);
 	} else {
 		gtk_widget_hide (dialog->description_vbox);
 	}
-	
+
 	str = gossip_vcard_get_name (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		gtk_label_set_text (GTK_LABEL (dialog->name_label), str);
 	}
-	
+
 	str = gossip_vcard_get_email (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		GtkWidget *href, *alignment;
 		gchar     *link;
 
@@ -315,10 +296,14 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 
 		link = g_strdup_printf ("mailto:%s", str);
 		href = gossip_link_button_new (link, str);
+		g_free (link);
+
+		/* Set widget names for style fixes */
+		gtk_widget_set_name (href, "gossip-contact-info-link-button");
 
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
-		
+
 		gtk_table_attach (GTK_TABLE (dialog->personal_table),
 				  alignment,
 				  1, 2,
@@ -326,8 +311,6 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 				  GTK_FILL, GTK_FILL,
 				  0, 0);
 
-		g_free (link);
-
 		gtk_widget_show_all (alignment);
 		gtk_widget_show (dialog->stub_email_label);
 	} else {
@@ -335,18 +318,21 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 	}
 
 	str = gossip_vcard_get_url (vcard);
-	if (str && strlen (str) > 0) {
+	if (!G_STR_EMPTY (str)) {
 		GtkWidget *href, *alignment;
 
 		show_personal = TRUE;
 
 		href = gossip_link_button_new (str, str);
 
+		/* Set widget names for style fixes */
+		gtk_widget_set_name (href, "gossip-contact-info-link-button");
+
 		alignment = gtk_alignment_new (0, 1, 0, 0.5);
 		gtk_container_add (GTK_CONTAINER (alignment), href);
 
 		gtk_table_attach (GTK_TABLE (dialog->personal_table),
-				  alignment, 
+				  alignment,
 				  1, 2,
 				  1, 2,
 				  GTK_FILL, GTK_FILL,
@@ -358,6 +344,32 @@ contact_info_dialog_get_vcard_cb (GossipResult   result,
 		gtk_widget_hide (dialog->stub_web_label);
 	}
 
+	str = gossip_vcard_get_birthday (vcard);
+	if (!G_STR_EMPTY (str)) {
+		GtkWidget *label, *alignment;
+
+		show_personal = TRUE;
+
+		label = gtk_label_new (str);
+		gtk_misc_set_padding (GTK_MISC (label), 1, -1);
+		gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+
+		alignment = gtk_alignment_new (0, 1, 0, 0.5);
+		gtk_container_add (GTK_CONTAINER (alignment), label);
+
+		gtk_table_attach (GTK_TABLE (dialog->personal_table),
+				  alignment,
+				  1, 2,
+				  2, 3,
+				  GTK_FILL, GTK_FILL,
+				  0, 0);
+
+		gtk_widget_show_all (alignment);
+		gtk_widget_show (dialog->stub_birthday_label);
+	} else {
+ 		gtk_widget_hide (dialog->stub_birthday_label); 
+	}
+
 	gtk_widget_hide (dialog->personal_status_hbox);
 
 	if (show_personal) {
@@ -376,7 +388,7 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 	const gchar             *str;
 	gboolean                 show_client_info = FALSE;
 
-	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
+	dialog = g_hash_table_lookup (dialogs, contact);
 	g_object_unref (contact);
 
 	if (!dialog) {
@@ -397,8 +409,10 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 		show_client_info = TRUE;
 
 		gtk_label_set_text (GTK_LABEL (dialog->client_label), str);
+		gtk_widget_show (dialog->client_label);
 		gtk_widget_show (dialog->stub_client_label);
 	} else {
+		gtk_widget_hide (dialog->client_label);
 		gtk_widget_hide (dialog->stub_client_label);
 	}
 
@@ -407,8 +421,10 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 		show_client_info = TRUE;
 
 		gtk_label_set_text (GTK_LABEL (dialog->version_label), str);
+		gtk_widget_show (dialog->version_label);
 		gtk_widget_show (dialog->stub_version_label);
 	} else {
+		gtk_widget_hide (dialog->version_label);
 		gtk_widget_hide (dialog->stub_version_label);
 	}
 
@@ -417,13 +433,17 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 		show_client_info = TRUE;
 
 		gtk_label_set_text (GTK_LABEL (dialog->os_label), str);
+		gtk_widget_show (dialog->os_label);
 		gtk_widget_show (dialog->stub_os_label);
 	} else {
+		gtk_widget_hide (dialog->os_label);
 		gtk_widget_hide (dialog->stub_os_label);
 	}
 
+	gtk_widget_hide (dialog->client_status_hbox);
+
 	if (show_client_info) {
-		gtk_widget_hide (dialog->client_status_hbox);
+		gtk_widget_show (dialog->client_vbox);
 		gtk_widget_show (dialog->client_table);
 	} else {
 		gtk_widget_hide (dialog->client_vbox);
@@ -431,94 +451,44 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 }
 
 static void
-contact_info_dialog_contact_updated_cb (GossipSession           *session,
-					GossipContact           *contact,
-					gpointer                 user_data)
-{
-	GossipContactInfoDialog *dialog;
-
-	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
-
-	if (!dialog) {
-		return;
-	}
-
-	if (!gossip_contact_equal (contact, dialog->contact)) {
-		return;
-	}
-	
-	contact_info_dialog_update_subscription (dialog);
-	contact_info_dialog_update_presences (dialog);
-}
-
-static void 
-contact_info_dialog_presence_updated_cb (GossipSession *session,
-					 GossipContact *contact,
+contact_info_dialog_presence_updated_cb (GossipContact *contact,
+					 GParamSpec    *param,
 					 gpointer       user_data)
 {
 	GossipContactInfoDialog *dialog;
 
-	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
+	dialog = g_hash_table_lookup (dialogs, contact);
 
 	if (!dialog) {
-		return;
-	}
-
-	if (!gossip_contact_equal (contact, dialog->contact)) {
+		g_signal_handlers_disconnect_by_func (contact,
+						      contact_info_dialog_presence_updated_cb,
+						      NULL);
 		return;
 	}
 
 	contact_info_dialog_update_presences (dialog);
 }
 
-static void
-contact_info_dialog_subscribe_clicked_cb (GtkWidget               *widget, 
-					  GossipContactInfoDialog *dialog)
-{
-	GossipSession *session;
-	GossipAccount *account;
-	const gchar   *message;
-
-	message = _("I would like to add you to my contact list.");
-
-	session = gossip_app_get_session ();
-	account = gossip_session_find_account (session, dialog->contact);
-
-        gossip_session_add_contact (session,
-				    account,
-                                    gossip_contact_get_id (dialog->contact), 
-				    gossip_contact_get_name (dialog->contact),
-				    NULL, /* group */
-				    message);
-
-	g_object_unref (account);
-}
-
 static void
 contact_info_dialog_destroy_cb (GtkWidget               *widget,
 				GossipContactInfoDialog *dialog)
 {
-	if (dialog->contact_signal_handler) {
-		g_signal_handler_disconnect (gossip_app_get_session (), 
-					     dialog->contact_signal_handler);
-	}
-
 	if (dialog->presence_signal_handler) {
-		g_signal_handler_disconnect (gossip_app_get_session (), 
+		g_signal_handler_disconnect (dialog->contact,
 					     dialog->presence_signal_handler);
 	}
 
 	if (dialog->show_timeout_id) {
 		g_source_remove (dialog->show_timeout_id);
 	}
-	
-	g_hash_table_remove (contact_info_dialogs, dialog->contact);
-	
+
+	g_hash_table_remove (dialogs, dialog->contact);
+
 	g_free (dialog);
 }
 
 static void
-contact_info_dialog_response_cb (GtkWidget               *widget, 
+contact_info_dialog_response_cb (GtkWidget               *widget,
 				 gint                     response,
 				 GossipContactInfoDialog *dialog)
 {
@@ -536,7 +506,8 @@ contact_info_dialog_show_timeout_cb (GossipContactInfoDialog *dialog)
 }
 
 void
-gossip_contact_info_dialog_show (GossipContact *contact)
+gossip_contact_info_dialog_show (GossipContact *contact,
+				 GtkWindow     *parent)
 {
 	GossipContactInfoDialog *dialog;
 	GossipSession           *session;
@@ -544,14 +515,16 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	GladeXML                *glade;
 	gchar                   *str;
 	GtkSizeGroup            *size_group;
-	guint                    id;
+	gulong                   id;
 	GtkWidget               *avatar_image_placeholder;
 	GdkPixbuf               *pixbuf;
 	gchar                   *path;
 
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
 	contact_info_dialog_init ();
 
-	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
+	dialog = g_hash_table_lookup (dialogs, contact);
 	if (dialog) {
 		gtk_window_present (GTK_WINDOW (dialog->dialog));
 		return;
@@ -564,7 +537,7 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 
 	dialog->contact = g_object_ref (contact);
 
-	g_hash_table_insert (contact_info_dialogs, dialog->contact, dialog);
+	g_hash_table_insert (dialogs, dialog->contact, dialog);
 
 	glade = gossip_glade_get_file ("main.glade",
 				       "contact_information_dialog",
@@ -582,6 +555,7 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 				       "client_table", &dialog->client_table,
 				       "stub_email_label", &dialog->stub_email_label,
 				       "stub_web_label", &dialog->stub_web_label,
+				       "stub_birthday_label", &dialog->stub_birthday_label,
 				       "stub_client_label", &dialog->stub_client_label,
 				       "stub_version_label", &dialog->stub_version_label,
 				       "stub_os_label", &dialog->stub_os_label,
@@ -590,13 +564,11 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 				       "client_status_label", &dialog->client_status_label,
 				       "client_status_hbox", &dialog->client_status_hbox,
 				       "client_vbox", &dialog->client_vbox,
-				       "subscription_hbox", &dialog->subscription_hbox,
-				       "subscription_label", &dialog->subscription_label,
 				       "presence_list_vbox", &dialog->presence_list_vbox,
 				       "presence_table", &dialog->presence_table,
 				       "avatar_image_placeholder", &avatar_image_placeholder,
 				       NULL);
-	
+
 	dialog->avatar_image = gossip_avatar_image_new (NULL);
 	gtk_container_add (GTK_CONTAINER (avatar_image_placeholder),
 			   dialog->avatar_image);
@@ -610,12 +582,11 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 		g_object_unref (pixbuf);
 	}
 	gtk_widget_show (dialog->avatar_image);
-	
+
 	gossip_glade_connect (glade,
 			      dialog,
 			      "contact_information_dialog", "destroy", contact_info_dialog_destroy_cb,
 			      "contact_information_dialog", "response", contact_info_dialog_response_cb,
-			      "subscribe_button", "clicked", contact_info_dialog_subscribe_clicked_cb,
 			      NULL);
 
 	g_object_unref (glade);
@@ -625,39 +596,36 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 
 	gtk_size_group_add_widget (size_group, dialog->stub_email_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_web_label);
+	gtk_size_group_add_widget (size_group, dialog->stub_birthday_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_client_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_version_label);
 	gtk_size_group_add_widget (size_group, dialog->stub_os_label);
 
 	g_object_unref (size_group);
-	
+
 	/* Set labels */
-	gtk_label_set_text (GTK_LABEL (dialog->id_label), 
+	gtk_label_set_text (GTK_LABEL (dialog->id_label),
 			    gossip_contact_get_id (contact));
 
-	gtk_label_set_text (GTK_LABEL (dialog->name_label), 
+	gtk_label_set_text (GTK_LABEL (dialog->name_label),
 			    gossip_contact_get_name (contact));
 
+	/* Set up transient parent */
+	if (parent) {
+		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog), parent);
+	}
+
 	/* Set details */
-	contact_info_dialog_update_subscription (dialog);
 	contact_info_dialog_update_presences (dialog);
-		
-	/* Subscription listener */
-	id = g_signal_connect (session,
-			       "contact-updated",
-			       G_CALLBACK (contact_info_dialog_contact_updated_cb), 
-			       NULL);
-	dialog->contact_signal_handler = id;
 
 	/* Presence listener */
-	id = g_signal_connect (session,
-			       "contact-presence-updated",
+	id = g_signal_connect (contact, "notify::presences",
 			       G_CALLBACK (contact_info_dialog_presence_updated_cb),
 			       NULL);
 	dialog->presence_signal_handler = id;
 
 	/* Get vcard and version info */
-	str = g_strdup_printf ("<i>%s</i>", 
+	str = g_strdup_printf ("<i>%s</i>",
 			       _("Information requested..."));
 	gtk_label_set_markup (GTK_LABEL (dialog->personal_status_label), str);
 	gtk_label_set_markup (GTK_LABEL (dialog->client_status_label), str);
@@ -670,15 +638,15 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 				  account,
 				  contact,
 				  (GossipVCardCallback) contact_info_dialog_get_vcard_cb,
-				  g_object_ref (contact), 
+				  g_object_ref (contact),
 				  NULL);
 
 	gossip_session_get_version (session,
 				    contact,
 				    (GossipVersionCallback) contact_info_dialog_get_version_cb,
-				    g_object_ref (contact), 
+				    g_object_ref (contact),
 				    NULL);
-	
+
 	dialog->show_timeout_id =
 		g_timeout_add (500,
 			       (GSourceFunc) contact_info_dialog_show_timeout_cb,
